const _ = require('lodash')
const path = require('path')
const fs = require('fs')
module.exports = async ({ strapi }) => {
  await bootstrapRolesAndPermissions({ strapi })
}

async function bootstrapRolesAndPermissions() {
  await strapi.db.lifecycles.subscribe({
    models: ['plugin::users-permissions.permission'],
    async afterCreate() {
      await writePermissionsFile()
    },
    async afterDelete() {
      await writePermissionsFile()
    },
  })

  try {
    let permissions = require(path.join(
      process.cwd(),
      'public',
      'permissions.json',
    ))
    await setDefaultPermissions(permissions, { strapi })
  } catch (e) {
    console.log(e)
  }
}
const writePermissionsFile = async () => {
  const PermJson = await getRolesWithPermissions()
  await fs.writeFile(
    path.join(process.cwd(), 'public', 'permissions.json'),
    JSON.stringify(PermJson,undefined,2),
    (err) => console.log(err),
  )
}
const getRolesWithPermissions = async () => {
  const service = strapi.plugins['users-permissions'].services.role

  const [roles] = await Promise.all([service.find()])

  const rolesWithPermissions = await Promise.all(
    roles.map(async (role) => await service.findOne(role.id)),
  )

  return rolesWithPermissions
}

async function setDefaultPermissions(_permissions, { strapi }) {
  const appRoles = await getRolesWithPermissions({ strapi })
  const service = strapi.plugins['users-permissions'].services.role

  let hasChanges = false
  main: for (let role of _permissions) {
    let remoteRole = await strapi
      .query('plugin::users-permissions.role')
      .findOne({ where: { type: role.type } })
    remoteRole = remoteRole&&await service.findOne(remoteRole.id)
    if (!remoteRole) {
      hasChanges = true
      break
    }

    const savedRole = role
    for (let [type, { controllers }] of Object.entries(role.permissions)) {
      for (let [controller, permissions] of Object.entries(controllers)) {
        for (let [action, { enabled }] of Object.entries(permissions)) {
          const savedPermission =
            savedRole.permissions[type].controllers[controller][action]
          if (!savedPermission) {
            continue
          }
          const remotePermission =
            remoteRole.permissions[type]?.controllers[controller]?.[action]

          if (!remotePermission) {
            continue
          }
          if (
            enabled &&
            !remotePermission?.permissions?.[type]?.controllers?.[controller]?.[action]?.enabled
          ) {
            hasChanges = true
            break main
          }
          if (
            !enabled &&
            remotePermission?.permissions?.[type]?.controllers?.[controller]?.[action]?.enabled
          ) {
            hasChanges = true
            break main
          }
        }
      }
    }
  }
  console.log({ hasChanges })
  if (hasChanges) {
    const service = strapi.plugins['users-permissions'].services.role
    const roleValues = Object.values(_permissions)
    await Promise.all(
      roleValues.map(async (data) => {
        const role = await strapi
          .query('plugin::users-permissions.role')
          .findOne({ where: { type: data.type } })
        const users = role ? role.users : []
        data.users = users

        if (!role) {
          await service.createRole(data)
        } else {
          await service.updateRole(role.id, data)
        }
      }),
    )
  }
}
